import os
import re

import authlib.integrations.base_client
import authlib.integrations.flask_client
import flask
import flask_sqlalchemy
import werkzeug.middleware.proxy_fix
from royalnet.typing import *

from ..database import Student
from ..database.base import Base
from ..deeplinking import DeepLinking


app = flask.Flask(__name__)
app.config.update(**os.environ)

reverse_proxy_app = werkzeug.middleware.proxy_fix.ProxyFix(app=app, x_for=1, x_proto=0, x_host=1, x_port=0, x_prefix=0)

db = flask_sqlalchemy.SQLAlchemy(app=app, metadata=Base.metadata)

oauth = authlib.integrations.flask_client.OAuth(app=app)
oauth.register(
    name="google",
    server_metadata_url='https://accounts.google.com/.well-known/openid-configuration',
    api_base_url="https://www.googleapis.com/",
    client_kwargs={
        "scope": "email profile openid",
    },
)

dl = DeepLinking(app.secret_key)


@app.route("/")
def page_index():
    return flask.render_template("info.html")


@app.route("/login")
def page_login():
    return oauth.google.authorize_redirect(flask.url_for("page_authorize", _external=True))


@app.route("/authorize")
def page_authorize():
    try:
        token = oauth.google.authorize_access_token()
    except werkzeug.exceptions.BadRequestKeyError:
        return flask.render_template(
            "error.html",
            error="Mancano i parametri query necessari per effettuare l'autenticazione OAuth.",
            tip='Torna all\'indice e rifai tutto da capo. Se il problema persiste, contatta '
                '<a href="https://t.me/Steffo">@Steffo</a>!'
        ), 401
    except authlib.integrations.base_client.errors.OAuthError:
        return flask.render_template(
            "error.html",
            error="Qualcosa è andato storto durante l'autenticazione.",
            tip='Torna all\'indice e rifai tutto da capo. Se il problema persiste, contatta '
                '<a href="https://t.me/Steffo">@Steffo</a>!'
        ), 401
    userinfo = oauth.google.parse_id_token(token=token)

    if not userinfo.email_verified:
        return flask.render_template(
            "error.html",
            error="L'email del tuo account Google non è verificata.",
            tip='Probabilmente hai effettuato l\'accesso con l\'email sbagliata. Fai il '
                '<a href="https://accounts.google.com/logout">logout</a> da tutti i tuoi account Google e riprova!'
        ), 403
    email_prefix_match = re.match(r"(.+)@studenti\.unimore\.it", userinfo.email)
    if not email_prefix_match:
        return flask.render_template(
            "error.html",
            error="Questo account Google non appartiene a Studenti Unimore Informatica.",
            tip='Probabilmente hai effettuato l\'accesso con l\'email sbagliata. Fai il '
                '<a href="https://accounts.google.com/logout">logout</a> da tutti i tuoi account Google e riprova!',
        ), 403
    email_prefix = email_prefix_match.group(1)

    student: Optional[Student] = db.session.query(Student).filter_by(email_prefix=email_prefix).one_or_none()
    if student is None:
        student = Student(
            email_prefix=email_prefix,
            first_name=userinfo.given_name,
            last_name=userinfo.family_name
        )
        db.session.add(student)
    else:
        student.first_name = userinfo.given_name
        student.last_name = userinfo.family_name
    db.session.commit()

    state = dl.encode(("R", student.email_prefix))
    if len(state) > 64:
        return flask.render_template("longer.html", state=state), 500

    return flask.redirect(f"https://t.me/{app.config['TELEGRAM_BOT_USERNAME']}?start={state}")


if __name__ == "__main__":
    db.create_all()
    app.run()



from __future__ import annotations

import abc

import royalnet.campaigns as rc
import telethon
import telethon.tl.custom as tc
from royalnet.typing import *
from telethon.hints import *

__all__ = (
    "Question",
    "Keyboard",
)


class ThorChallenge(rc.AsyncChallenge, metaclass=abc.ABCMeta):
    """
    Like a Royalnet challenge, but also define an action to be taken by the TelegramClient if this becomes the new
    challenge.
    """

    @abc.abstractmethod
    async def send(self, bot: telethon.TelegramClient, entity: Entity) -> ThorChallenge:
        raise NotImplementedError()


class Question(ThorChallenge):
    """Send a message to the chat, and wait for anything."""

    def __init__(self, message: str, *message_args, **message_kwargs):
        super().__init__()
        self.message: str = message
        self.message_args = message_args
        self.message_kwargs = message_kwargs

    async def send(self, bot: telethon.TelegramClient, entity: Entity) -> Question:
        await bot.send_message(
            entity=entity,
            message=self.message,
            parse_mode="HTML",
            *self.message_args,
            **self.message_kwargs)
        return self

    async def filter(self, data: Any) -> bool:
        return True


class UnrestrictedKeyboard(Question):
    """Send a message and a custom keyboard to the chat, and wait for anything."""

    def __init__(self, message: str, choices: List[List[str]], *args, **kwargs):
        super().__init__(message, *args, **kwargs)
        self.choices: List[List[str]] = choices

    def flat_choices(self) -> List[str]:
        """
        :return: The flattened choices list.
        """
        li = []
        for row in self.choices:
            for choice in row:
                li.append(choice)
        return li

    def buttons(self) -> List[List[ButtonLike]]:
        """
        :return: The list of choices, converted to telethon button objects.
        """
        new_rows = []
        for row in self.choices:
            new_row = []
            for choice in row:
                new_row.append(telethon.Button.text(choice))
            new_rows.append(new_row)
        return new_rows

    async def send(self, bot: telethon.TelegramClient, entity: Entity) -> UnrestrictedKeyboard:
        markup = bot.build_reply_markup(self.buttons())
        await bot.send_message(
            entity=entity,
            message=self.message,
            buttons=markup,
            parse_mode="HTML",
            *self.message_args,
            **self.message_kwargs)
        return self


class Keyboard(UnrestrictedKeyboard):
    """Send a message and a custom keyboard to the chat, and only allow one of the possible keyboard responses."""

    async def filter(self, data: tc.Message) -> bool:
        return data.message and data.message in self.flat_choices()



import asyncio
import logging
import os

import coloredlogs
import royalnet.alchemist
import royalnet.campaigns
import telethon
import telethon.tl.custom
import sys
import traceback
from royalnet.typing import *

from .dialog import Dialog
from ..database import Telegram
from ..database.base import Base

log = logging.getLogger(__name__)

log.info("Creating Alchemist...")
alchemist: royalnet.alchemist.Alchemist = royalnet.alchemist.Alchemist(
    engine_args=[os.environ["SQLALCHEMY_DATABASE_URI"]],
    engine_kwargs={}
)
log.debug("Mapping database tables...")
alchemist.add_metadata(Base.metadata)

log.debug("Creating telethon TelegramClient...")
client = telethon.client.TelegramClient("bot", int(os.environ["TELEGRAM_API_ID"]), os.environ["TELEGRAM_API_HASH"])


async def main():
    logging.root.setLevel(os.environ["LOG_LEVEL"])
    stream_handler = logging.StreamHandler()
    stream_handler.formatter = coloredlogs.ColoredFormatter(os.environ["LOG_FORMAT"].replace("\\t", "\t"), style="{")
    logging.root.addHandler(stream_handler)
    log.debug("Logging setup successfully!")

    log.debug("Starting telethon TelegramClient...")
    # noinspection PyProtectedMember
    bot = await client._start(
        phone=None,
        password=None,
        bot_token=os.environ["TELEGRAM_BOT_TOKEN"],
        force_sms=False,
        code_callback=None,
        first_name='Thor Bot',
        last_name='',
        max_attempts=3)

    me = await bot.get_me()
    log.debug(f"Logged in as: {me.first_name} <{me.id}>")

    menus: Dict[int, Dialog] = {}

    @bot.on(telethon.events.ChatAction())
    async def on_chat_action(event: telethon.events.ChatAction.Event):
        if event.user_joined:
            users = await event.get_users()
            chat = await event.get_chat()
            assert len(users) == 1
            user = users[0]
            session = alchemist.Session()
            tg = session.query(Telegram).filter_by(id=user.id).one_or_none()
            if tg is None:
                await bot.kick_participant(entity=chat, user=user)
                await bot.send_message(
                    entity=chat,
                    parse_mode="HTML",
                    message="🚫 L'account è stato rimosso dal gruppo perchè non autenticato."
                )
            else:
                await bot.send_message(
                    entity=chat,
                    parse_mode="HTML",
                    message=tg.st.whois()
                )

    @bot.on(telethon.events.NewMessage())
    async def on_message(event: telethon.events.NewMessage.Event):
        msg: telethon.tl.custom.Message = event.message
        log.debug(f"Received message: {msg}")
        if msg.chat_id not in menus:
            log.debug(f"Creating a new Menu for {msg.chat_id}")
            menus[msg.chat_id] = await Dialog.create(bot=bot, entity=msg.chat, session=alchemist.Session())
        menu = menus[msg.chat_id]
        try:
            await menu.next(msg)
        except StopAsyncIteration:
            del menus[msg.chat_id]
        except Exception as e:
            log.error("".join(traceback.format_exception(*sys.exc_info())))
            await bot.send_message(
                entity=msg.chat,
                message="☢️ Si è verificato un errore critico e la conversazione è stata annullata.\n"
                        "\n"
                        "L'errore è stato salvato nei log del server.")

    while True:
        log.info(f"Catching up...")
        await bot.catch_up()

        log.info(f"Running!")
        # noinspection PyProtectedMember
        await bot._run_until_disconnected()


if __name__ == "__main__":
    asyncio.get_event_loop().run_until_complete(main())



from royalnet.typing import *
import sqlalchemy as s
import sqlalchemy.orm as o
import html

from .base import Base


class Telegram(Base):
    """
    A table that contains data related to the second step of the student verification process (Telegram menu).
    """
    __tablename__ = "telegram"

    id = s.Column(s.BigInteger, primary_key=True)
    first_name = s.Column(s.String, nullable=False)
    last_name = s.Column(s.String)
    username = s.Column(s.String)

    st_email_prefix = s.Column(s.String, s.ForeignKey("students.email_prefix"), nullable=False)
    st = o.relationship("Student", back_populates="tg", uselist=False)

    def __repr__(self):
        return f"{self.__qualname__}({self.id=}, {self.first_name=}, {self.last_name=}, {self.username=}, " \
               f"{self.st_email_prefix=})"

    def __str__(self):
        """
        :return: The Telegram account's full name
        """
        if self.last_name:
            return f"{self.first_name} {self.last_name}"
        else:
            return self.first_name

    def name_mention(self) -> str:
        """
        Create a Telegram name mention.

        :return: The mention.
        """
        return f'<a href="tg://user?id={self.id}">{html.escape(str(self))}</a>'

    def at_mention(self) -> Optional[str]:
        """
        Create a Telegram username mention.

        :return: The mention, or None if the user has no username.
        """
        if self.username:
            return f"@{self.username}"
        else:
            return None

    def whois(self) -> str:
        """Compose the whois message for this student, respecting privacy settings."""
        return self.st.whois()

    def whois_message(self) -> str:
        """Compose the whois message for this student, ignoring privacy settings."""
        return self.st.whois_message()

    def minimessage(self) -> str:
        """Compose the whois message subsection for this Telegram account."""
        newline = "\n"
        return f"📱 {self.name_mention()}\n" \
               f"{self.at_mention() + newline or ''}"



import itsdangerous
from royalnet.typing import *


class DeepLinking:
    """A helper class to pass secure information between Telegram and Flask via Telegram Deep Linking."""

    def __init__(self, secret_key: str, namespace: str = "t"):
        self.serializer = itsdangerous.URLSafeSerializer(secret_key=secret_key, salt=namespace)

    def encode(self, value: Any) -> str:
        signed = self.serializer.dumps(value)
        signed = signed.replace("_", "__").replace(".", "_")
        return signed

    def decode(self, signed: str) -> Any:
        signed = signed.replace("_", ".").replace("..", "_")
        value = self.serializer.loads(signed)
        return value



import sqlalchemy as s
import sqlalchemy.orm as o

from .base import Base


class Student(Base):
    """
    A table that contains data related to the first step of the student verification process (Google sign in).
    """
    __tablename__ = "students"

    email_prefix = s.Column(s.String, nullable=False, primary_key=True)
    first_name = s.Column(s.String, nullable=False)
    last_name = s.Column(s.String, nullable=False)

    privacy = s.Column(s.Boolean, nullable=False, default=True, server_default="TRUE")
    """Whether or not the student has requested to keep his data hidden from all users."""

    tg = o.relationship("Telegram", back_populates="st")

    def email(self):
        """
        Compose the full email of the student.

        :return: The email in form of a str.
        """
        return f"{self.email_prefix}@studenti.unimore.it"

    def __str__(self) -> str:
        """
        :return: The full name of the user and their email in classic form (Stefano Pigozzi <ste.pigozzi@gmail.com>).
        """
        return f"{self.first_name} {self.last_name} <{self.email()}>"

    def __repr__(self):
        return f"{self.__qualname__}({self.email_prefix=}, {self.first_name=}, {self.last_name=}, {self.privacy=})"

    def whois(self) -> str:
        """
        Compose the whois message for this student, respecting privacy settings.

        :return: The composed message.
        """
        if self.privacy:
            return "👤 Lo studente è registrato, ma ha deciso di manterere privati i dettagli del suo account."
        return self.whois_message()

    def whois_message(self):
        """
        Compose the whois message for this student, ignoring privacy settings.

        :return: The composed message.
        """
        rows = [
            f"🎓 <b>{self.first_name} {self.last_name}</b>",
            f"{self.email()}",
            ""
        ]

        tgs = [tg for tg in self.tg]
        for tg in tgs:
            rows.append(tg.minimessage())
            rows.append("")

        return "\n".join(rows)



from __future__ import annotations

import logging
import os
import re

import itsdangerous
import royalnet.campaigns
import royalnet.campaigns.exc
import sqlalchemy.orm
import telethon
import telethon.tl.custom
from royalnet.typing import *
from telethon.hints import *

from .challenges import *
from ..database import Student, Telegram
from ..deeplinking import DeepLinking

log = logging.getLogger(__name__)
dl = DeepLinking(os.environ["SECRET_KEY"])


email_regex = re.compile(r"^([0-9]+)(?:@studenti\.unimore\.it)?$")


class Dialog:
    def __init__(self, bot: telethon.TelegramClient, entity: Entity, session: sqlalchemy.orm.Session):
        """
        Initialize an Dialog object.

        .. warning:: Do not use this, use the Dialog.create() method instead!
        """
        self.bot: telethon.TelegramClient = bot
        "The bot at one side of the Dialog."

        self.entity: Entity = entity
        "The entity (user, group) at the other side of the Dialog."

        self.session: sqlalchemy.orm.Session = session
        "The SQLAlchemy Session to be used exclusively for this dialog."

        self.campaign: royalnet.campaigns.AsyncCampaign = ...
        """
        The AsyncCampaign this dialog will use to track the back and forth (taking form of multiple AsyncAdventures).
        
        May be an instance of Ellipsis if the Dialog is not initialized.
        """

    @classmethod
    async def create(cls,
                     bot: telethon.TelegramClient,
                     entity: Entity,
                     session: sqlalchemy.orm.Session) -> Dialog:
        """
        Create a new Dialog object.

        :param bot: The bot at one side of the Dialog.
        :param entity: The entity (user, group) at the other side of the Dialog.
        :param session: The SQLAlchemy Session to be used exclusively for this dialog.
        :return: The created dialog.
        """
        menu = cls(bot=bot, entity=entity, session=session)
        menu.campaign = await royalnet.campaigns.AsyncCampaign.create(start=menu.__first())
        return menu

    async def next(self, msg: telethon.tl.custom.Message) -> None:
        """
        Advance to the next message of the Dialog.

        :param msg: The message to pass to the AsyncAdventure.
        """
        try:
            log.debug(f"Advancing: {self}")
            await self.campaign.next(msg)
        except royalnet.campaigns.exc.ChallengeFailedError:
            log.debug(f"Challenge failed: {self}")
            return
        except StopAsyncIteration:
            log.debug(f"Closing: {self}")
            self.session.close()
            raise
        else:
            log.debug(f"Sending: {self.campaign.challenge}")
            await self.campaign.challenge.send(bot=self.bot, entity=self.entity)

    async def __message(self, msg, **kwargs) -> telethon.types.Message:
        """
        Send a Telegram message to the user at the other side of the dialog.

        It's a shortcut to self.bot.send_message.

        :param msg: The contents of the message.
        :param kwargs: Keyword arguments to pass to send_message.
        :return: The Message returned by send_message.
        """
        return await self.bot.send_message(
            entity=self.entity,
            parse_mode="HTML",
            message=msg,
            buttons=self.bot.build_reply_markup(telethon.tl.custom.Button.clear()),
            **kwargs
        )

    async def __first(self) -> AsyncAdventure:
        """What to do when the dialog is first opened."""
        msg: telethon.tl.custom.Message = yield

        # If this is a text message
        if text := msg.message:
            if text.startswith("/whois"):
                yield self.__whois()

            elif text.startswith("/start"):
                if msg.is_private:
                    yield self.__start()
                else:
                    await self.__message(
                        f"⚠️ Questo comando funziona solo in chat privata (@{os.environ['TELEGRAM_BOT_USERNAME']})."
                    )

            elif text.startswith("/privacy"):
                if msg.is_private:
                    yield self.__privacy()
                else:
                    await self.__message(
                        f"⚠️ Questo comando funziona solo in chat privata (@{os.environ['TELEGRAM_BOT_USERNAME']})."
                    )

    async def __start(self) -> AsyncAdventure:
        """Disambiguation for the /start command."""
        msg: telethon.tl.custom.Message = yield

        text: str = msg.message

        # Check if the user is already registered
        from_user = await msg.get_sender()
        tg: Telegram = self.session.query(Telegram).filter_by(id=from_user.id).one_or_none()
        if tg is not None:
            await self.__message(
                f'⭐️ Hai già effettuato la verifica dell\'identità.\n\n'
                f'<a href="{os.environ["GROUP_URL"]}">Entra nel gruppo cliccando '
                f'qui!</a>'
            )
            return

        # Check whether this is a normal start or a deep-linked one
        split = text.split(" ", 1)
        if len(split) == 1:
            yield self.__normal_start()
        else:
            yield self.__deeplink_start(payload=split[1])

    async def __normal_start(self) -> AsyncAdventure:
        """The /start command, called without arguments."""
        msg: telethon.tl.custom.Message = yield

        await self.__message(
            f'👋 Ciao! Sono Thor, il bot-moderatore di Unimore Informatica.\n'
            f'\n'
            f'Se vuoi entrare nel gruppo, devi <b>dimostrare di essere uno studente dell\'Unimore</b>.\n'
            f'\n'
            f'<a href="{os.environ["BASE_URL"]}">Fai il login con il tuo account universitario qui</a>, poi una volta '
            f'tornato su Telegram premi il tasto <i>AVVIA</i> in basso per ricevere il link! 😊'
        )

    async def __deeplink_start(self, payload: str) -> AsyncAdventure:
        """The /start command, called with deep-linked arguments."""
        msg: telethon.tl.custom.Message = yield

        try:
            opcode, data = dl.decode(payload)
        except itsdangerous.exc.BadData:
            await self.__message("⚠️ I dati ricevuti non sono validi.")
            return

        # R: Register new account
        if opcode == "R":
            yield self.__register(email_prefix=data)
        else:
            await self.__message("⚠️ Ricevuto un opcode sconosciuto.")

    async def __register(self, email_prefix: str) -> AsyncAdventure:
        """
        The /start command, called with a payload starting with R.

        Links a Telegram account to a real student.

        :param email_prefix: The prefix before @studenti.unimore.it
        """
        msg: telethon.tl.custom.Message = yield

        from_user = await msg.get_sender()

        tg: Telegram = self.session.query(Telegram).filter_by(id=from_user.id).one_or_none()
        st: Student = self.session.query(Student).filter_by(email_prefix=email_prefix).one()

        # Ask for confirmation
        choice = yield Keyboard(
            message=f'❔ Tu sei {st.first_name} {st.last_name} <{st.email()}>, giusto?',
            choices=[["❌ No.", "✅ Sì!"]]
        )
        if choice.message == "❌ No.":
            await self.__message(
                "↩️ Effettua il logout da tutti gli account Google sul tuo browser, poi ri-invia il comando /start!"
            )
            return

        # Ask for privacy mode
        choice = yield Keyboard(
            message="📝 Vuoi permettere agli altri studenti verificati di visualizzare il tuo <b>vero nome</b> e la tua "
                    "<b>email istituzionale</b> attraverso il comando /whois?\n\n"
                    "(Gli amministratori del gruppo vi avranno comunque accesso, e potrai cambiare idea in qualsiasi "
                    "momento con il comando /privacy.)",
            choices=[["👤 Nascondi.", "📱 Mostra!"]]
        )
        st.privacy = choice.message == "👤 Nascondi."

        # Create the SQL record
        tg = Telegram(
            id=from_user.id,
            first_name=from_user.first_name,
            last_name=from_user.last_name,
            username=from_user.username,
            st=st,
        )

        # Commit the SQLAlchemy session
        self.session.add(tg)
        self.session.commit()

        # Send the link to the group
        await self.__message(
            f'✨ Hai completato la verifica dell\'identità.\n\n'
            f'<a href="{os.environ["GROUP_URL"]}">Entra nel gruppo cliccando qui!</a>'
        )
        return

    async def __privacy(self) -> AsyncAdventure:
        """The /privacy command, used to toggle between privacy states."""
        msg: telethon.tl.custom.Message = yield

        from_user = await msg.get_sender()

        tg: Telegram = self.session.query(Telegram).filter_by(id=from_user.id).one_or_none()
        if tg is None:
            await self.__message(
                "⚠️ Non hai ancora effettuato la verifica dell'account!\n"
                "\n"
                "Invia /start per iniziare!"
            )
            return

        # Ask for privacy mode
        choice = yield Keyboard(
            message="📝 Vuoi permettere agli altri studenti verificati di visualizzare il tuo <b>vero nome</b> e la tua "
                    "<b>email istituzionale</b> attraverso il comando /whois?\n\n"
                    "(Gli amministratori del gruppo vi avranno comunque accesso, e potrai cambiare idea in qualsiasi "
                    "momento con il comando /privacy.)",
            choices=[["👤 Nascondi.", "📱 Mostra!"]],
        )
        tg.st.privacy = choice.message == "👤 Nascondi."
        self.session.commit()

        if tg.st.privacy:
            await self.__message("👤 I tuoi dati ora sono nascosti.")
        else:
            await self.__message("📱 I tuoi dati ora sono visibili attraverso il comando /whois!")

    async def __whois(self) -> AsyncAdventure:
        """The /whois command, used to fetch information about a certain student or Telegram account."""
        msg: telethon.tl.custom.Message = yield

        cmd, *args = msg.message.split(" ", 1)
        args = " ".join(args)

        # Email
        if match := re.match(email_regex, args):
            email_prefix = match.group(1)
            yield self.__whois_email(email_prefix=email_prefix)

        # Real name
        elif " " in args:
            yield self.__whois_real_name(name=args)

        elif args.startswith("@"):
            username = args.lstrip("@")
            yield self.__whois_username(username=username)

        # TODO: Match telegram mentions
        # TODO: Match telegram name

        await self.__message(
            "⚠️ Non hai specificato correttamente cosa cercare.\n"
            "\n"
            "Puoi specificare un'username Telegram, un nome e cognome o un'email.\n"
            "<code>/whois Stefano Pigozzi</code>\n"
            "<code>/whois @Steffo</code>\n"
            "<code>/whois 256895@studenti.unimore.it</code>\n"
            "\n"
            "🚧 La funzionalità di ricerca tramite name-mention di Telegram non è ancora stata implementata."
        )

    async def __whois_email(self, email_prefix: str):
        """The /whois command, called with an email."""
        msg: telethon.tl.custom.Message = yield

        st: Optional[Student] = self.session.query(Student).filter_by(email_prefix=email_prefix).one_or_none()
        if st is None:
            await self.__message("⚠️ Nessuno studente trovato.")
            return

        await self.__message(st.whois())

    async def __whois_real_name(self, name: str) -> AsyncAdventure:
        """The /whois command, called with a first name and a last name."""
        msg: telethon.tl.custom.Message = yield

        students = (
            self.session
            .query(Student)
            .filter(
                sqlalchemy.or_(
                    sqlalchemy.func.concat(Student.first_name, " ", Student.last_name) == name.upper(),
                    sqlalchemy.func.concat(Student.last_name, " ", Student.first_name) == name.upper(),
                )
            )
            .all()
        )

        if len(students) == 0:
            await self.__message("⚠️ Nessuno studente trovato.")
            return

        # There might be more than a student with the same name!
        response: List[str] = []
        for student in students:
            response.append(student.whois())

        await self.__message("\n\n".join(response))

    async def __whois_username(self, username: str) -> AsyncAdventure:
        """The /whois command, called with a Telegram username."""
        msg: telethon.tl.custom.Message = yield

        tg: Optional[Telegram] = self.session.query(Telegram).filter_by(username=username).one_or_none()
        if tg is None:
            await self.__message("⚠️ Nessuno studente trovato.")
            return

        await self.__message(tg.st.whois())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['thorunimore',
 'thorunimore.database',
 'thorunimore.database.base',
 'thorunimore.telegram',
 'thorunimore.web']

package_data = \
{'': ['*'], 'thorunimore.web': ['static/*', 'templates/*']}

install_requires = \
['aiohttp>=3.6.2,<4.0.0',
 'authlib>=0.14.3,<0.15.0',
 'coloredlogs>=14.0,<15.0',
 'flask-sqlalchemy>=2.4.4,<3.0.0',
 'flask>=1.1.2,<2.0.0',
 'itsdangerous>=1.1.0,<2.0.0',
 'requests>=2.24.0,<3.0.0',
 'royalnet>=6.0.0a1,<7.0.0',
 'telethon>=1.16.4,<2.0.0']

setup_kwargs = {
    'name': 'thorunimore',
    'version': '1.0.7',
    'description': 'A moderator bot for the Unimore Informatica group',
    'long_description': None,
    'author': 'Stefano Pigozzi',
    'author_email': 'ste.pigozzi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
